﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TaskScheduler;

namespace EventReminder
{
    
    class TaskManager
    {
        public static bool CreateTask(string TaskName, string Author, string ExePath, string Arguments, short DaysInterval, int Hours, int Minutes, bool bShowErrorMessage)
        {
            if (!System.IO.File.Exists(ExePath))
            {
                System.Windows.Forms.MessageBox.Show("Error Creating Scheduled Task : " + ExePath + " doesn't exists",
                                                    "Error Creating Scheduled Task",
                                                    System.Windows.Forms.MessageBoxButtons.OK,
                                                    System.Windows.Forms.MessageBoxIcon.Error);

                return false;
            }

            try
            {

                TaskScheduler.TaskScheduler Service = new TaskScheduler.TaskScheduler();
                Service.Connect(null, null, null, null);

                // get root folder
                ITaskFolder RootFolder = Service.GetFolder("");

                try // avoid to throw error if task is not existing
                {
                    // delete task if already existing
                    RootFolder.DeleteTask(TaskName, 0);
                }
                catch (System.Exception ex)
                {
                	
                }


                ITaskDefinition Task = Service.NewTask(0);
                Task.RegistrationInfo.Author = Author;
                Task.RegistrationInfo.Description = TaskName;

                IPrincipal Principal = Task.Principal;
                Principal.LogonType = _TASK_LOGON_TYPE.TASK_LOGON_INTERACTIVE_TOKEN;

                ITaskSettings Settings = Task.Settings;
                Settings.StartWhenAvailable = true;

                IDailyTrigger Trigger = (IDailyTrigger)Task.Triggers.Create(_TASK_TRIGGER_TYPE2.TASK_TRIGGER_DAILY);
                Trigger.Id = "DailyTrigger";
                Trigger.DaysInterval = DaysInterval;

                Trigger.StartBoundary = "2010-01-01T" + Hours.ToString("D2") + ":" + Minutes.ToString("D2") + ":00";
                Trigger.EndBoundary = "2100-12-31T12:00:00";

                IExecAction Action = (IExecAction)Task.Actions.Create(_TASK_ACTION_TYPE.TASK_ACTION_EXEC);
                Action.Path = ExePath;
                Action.Arguments = Arguments;


                IRegisteredTask regTask = RootFolder.RegisterTaskDefinition(
                                            TaskName,
                                            Task,
                                            (int)_TASK_CREATION.TASK_CREATE_OR_UPDATE,
                                            null, //user
                                            null, // password
                                            _TASK_LOGON_TYPE.TASK_LOGON_INTERACTIVE_TOKEN,
                                            "");
                return true;
            }
            catch (System.Exception ex)
            {
                if (bShowErrorMessage)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message,
                                                        "Error Creating Scheduled Task",
                                                        System.Windows.Forms.MessageBoxButtons.OK,
                                                        System.Windows.Forms.MessageBoxIcon.Error);
                }
                return false;
            }
        }



        public static bool DeleteTask(string TaskName)
        {
            try
            {

                TaskScheduler.TaskScheduler Service = new TaskScheduler.TaskScheduler();
                Service.Connect(null, null, null, null);

                // get root folder
                ITaskFolder RootFolder = Service.GetFolder("");

                // delete task if already existing
                RootFolder.DeleteTask(TaskName, 0);

                return true;
            }
            catch (System.Exception ex)
            {
                // avoid to show error if task is not existing
                //System.Windows.Forms.MessageBox.Show(ex.Message,
                //                                    "Error",
                //                                    System.Windows.Forms.MessageBoxButtons.OK,
                //                                    System.Windows.Forms.MessageBoxIcon.Error);
                return false;
            }
        }
    }
}

