/******************************************************************************************************************
*	Author:				Jacquelin POTIER
*	Date:				10/31/2002
*	Copyright:			 2002, Jacquelin POTIER (jacquelin.potier@libertysurf.fr). 
*                                       Free for non commercial use (please let author's name)
/******************************************************************************************************************/
using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;

namespace EventReminder
{
#region IShellLink
    [ComImport]
    [Guid("00021401-0000-0000-C000-000000000046")]
    internal class ShellLink
    {
    }

    [ComImport]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("000214F9-0000-0000-C000-000000000046")]
    internal interface IShellLink
    {
        void GetPath([Out, MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszFile, int cchMaxPath, out IntPtr pfd, int fFlags);
        void GetIDList(out IntPtr ppidl);
        void SetIDList(IntPtr pidl);
        void GetDescription([Out, MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszName, int cchMaxName);
        void SetDescription([MarshalAs(UnmanagedType.LPWStr)] string pszName);
        void GetWorkingDirectory([Out, MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszDir, int cchMaxPath);
        void SetWorkingDirectory([MarshalAs(UnmanagedType.LPWStr)] string pszDir);
        void GetArguments([Out, MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszArgs, int cchMaxPath);
        void SetArguments([MarshalAs(UnmanagedType.LPWStr)] string pszArgs);
        void GetHotkey(out short pwHotkey);
        void SetHotkey(short wHotkey);
        void GetShowCmd(out int piShowCmd);
        void SetShowCmd(int iShowCmd);
        void GetIconLocation([Out, MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszIconPath, int cchIconPath, out int piIcon);
        void SetIconLocation([MarshalAs(UnmanagedType.LPWStr)] string pszIconPath, int iIcon);
        void SetRelativePath([MarshalAs(UnmanagedType.LPWStr)] string pszPathRel, int dwReserved);
        void Resolve(IntPtr hwnd, int fFlags);
        void SetPath([MarshalAs(UnmanagedType.LPWStr)] string pszFile);
    }
#endregion


    /// <summary>
    /// Description rsume de main_engine.
    /// </summary>
    public class class_main
    {
        public class_options options;
        public class_agent clagent;
        private class_events_to_remind clevents_to_remind;
        public class_welcome_messages clwelcome_messages;
#if OLD_OS_CONFIG
        [System.Runtime.InteropServices.DllImport("Easy_task.dll", EntryPoint = "set_application", CallingConvention = System.Runtime.InteropServices.CallingConvention.Cdecl)]
            static extern int Easy_task_set_application(String task_name,String application_name);//(char* str);
        [System.Runtime.InteropServices.DllImport("Easy_task.dll", EntryPoint = "enable_task_for_logged_user", CallingConvention = System.Runtime.InteropServices.CallingConvention.Cdecl)]
            static extern int Easy_task_enable_task_for_logged_user(String task_name);//(char* str);
        [System.Runtime.InteropServices.DllImport("Easy_task.dll", EntryPoint = "set_daily_trigger_with_day_interval", CallingConvention = System.Runtime.InteropServices.CallingConvention.Cdecl)]
            static extern int Easy_task_set_daily_trigger(String task_name,ushort start_hour,ushort start_min,ushort days_interval);//(char* str);
        [System.Runtime.InteropServices.DllImport("Easy_task.dll", EntryPoint = "delete_task", CallingConvention = System.Runtime.InteropServices.CallingConvention.Cdecl)]
			static extern int Easy_task_delete(String task_name);//(char* str);
#endif
        public class_main()
        {
			options=class_options.load();
        }


        #region agent management

        public void start_agent()
        {
			options=class_options.load();
            clagent=new class_agent();
            load_agent_and_apply_settings();
        }

        public void shutdown()
        {
            clagent.unload_agent();
        }

        /// <summary>
        /// load agent and apply options setting
        /// </summary>
        public void load_agent_and_apply_settings()
        {
            clagent.load(options.agent_name,false);
            clagent.show_ballon(options.ballon_on);
            clagent.chrtag=options.voice_character;
            clagent.ctxtag=options.voice_context;
            clagent.default_voice=options.default_voice;
            clagent.pittag=options.voice_pitch;
            clagent.spdtag=options.voice_speed;
            switch (options.engine_type)
            {
                case class_options.tag_engine_type.engine_type_language:
                    clagent.change_language(options.language);
                    break;
                case class_options.tag_engine_type.engine_type_TTS_SAPI4:
                    clagent.change_tts_engine_from_description(options.tts_engine);
                    break;
                case class_options.tag_engine_type.engine_type_TTS_SAPI5:
                    clagent.change_tts_engine(options.SAPI5_tts_engine);
                    break;
            }
        }
        #endregion

        #region envents checking
        /// <summary>
        /// /run mode
        /// </summary>
        public void event_dates_check()
        {
            event_dates_check(true);
        }

        private void event_dates_check(bool close_after_checking)
        {

            // load events and messages
            clevents_to_remind=new class_events_to_remind();
            clevents_to_remind.load();
            clwelcome_messages=new class_welcome_messages();
            clwelcome_messages.load();

            //start agent
            start_agent();

            if (close_after_checking) // only if in run mode else let agent in initial position
                // position agent
                play_random_move();

            // show the agent
            clagent.Show(false);

            // speak welcome message
            if (options.play_welcome_message)
                clagent.speak(get_random_welcome_message());

            // make a search animation
            play_random_search();

            // checking events
            System.DateTime now_date=System.DateTime.Now; // get current time
            // set hour, minutes seconds ... to 0 to avoid error with System.DateTime.Subtract
            now_date=new System.DateTime(now_date.Year,now_date.Month,now_date.Day);

            int current_year=now_date.Year;
            System.Collections.ArrayList al_ordered_events=new System.Collections.ArrayList (3);//arraylist of single_event_with_day_remaining
            int remaining_days;
            System.TimeSpan sub;
            int cpt;
            bool inserted;
            int size;

            // find events to remind
            foreach(single_event se in clevents_to_remind.al_events)
            {
                // comput date_i-date_now
                sub=new System.DateTime(current_year,se.date.month,se.date.day).Subtract(now_date);
                // get remaining days
                remaining_days=sub.Days;
                if ((remaining_days>=0) && (remaining_days<=se.day_before))
                {
                    if (al_ordered_events.Count==0) // event is today
                    {
                        al_ordered_events.Add(new single_event_with_day_remaining(se,remaining_days));
                    }
                    else // event is in n days
                    {
                        // order events by days remaining
                        inserted=false;
                        size=al_ordered_events.Count;
                        for(cpt=0;cpt<size;cpt++)
                        {
                            if (((single_event_with_day_remaining)al_ordered_events[cpt]).day_remaining>remaining_days)
                            {
                                al_ordered_events.Insert(cpt,new single_event_with_day_remaining(se,remaining_days));
                                inserted=true;
                                break;
                            }
                        }
                        if (!inserted)
                            // add at the end of arraylist
                            al_ordered_events.Add(new single_event_with_day_remaining(se,remaining_days));
                    }
                }
            }
            // get information to speak/show

            string text_to_speak="";
            bool information_to_show=false;
            string strtmp;
            FormEventWarning few=new FormEventWarning();

            foreach(single_event_with_day_remaining ser in al_ordered_events)
            {

                // case of day remaining
                if (ser.day_remaining==0)
                {
                    information_to_show=true;
                    // prepare speak message_to_read
                    text_to_speak+=ser.message_to_read+".\r\n";// add point for prononciation
                    // prepare information to show (add controls to the few window)
                    few.add_event(ser.name,"phone: "+ser.phone,"e-mail: "+ ser.email);
                }
                else
                {
                    // prepare speak 
                    strtmp=options.n_days_before_event;
                    strtmp=strtmp.Replace("_n_days",ser.day_remaining.ToString());
                    strtmp=strtmp.Replace("_event",ser.event_description);
                    text_to_speak+=strtmp+".\r\n";
                }
            }

            // say welcome message if wanted
            if ((options.play_message_if_noevent_at_windows_startup) && (al_ordered_events.Count==0))
                text_to_speak=options.no_event_speech;


            // show informations if needed
            if (information_to_show)
            {
                play_random_event_animation();

                // make agent speak
                clagent.speak(text_to_speak,close_after_checking);

                // get windows size
                System.Drawing.Rectangle working_area = System.Windows.Forms.SystemInformation.WorkingArea;
                // show window
                few.AnchorPoint=new System.Drawing.Point(working_area.Right,working_area.Bottom);
                few.close_FormMain_onclose(close_after_checking);// if (close_after_checking) few.close_FormMain_onclose(true);
                // if close_after_checking wait the agent is away to show ballon
                if (!close_after_checking) few.Show();
            }
            else
            {
                if (al_ordered_events.Count>0)
                {
                    play_random_attention_animation();
                }
                else
                {
                    play_random_no_event_animation();
                }
                

                // make agent speak
                clagent.speak(text_to_speak,close_after_checking);

            }
            if (close_after_checking) // if we are in /run mode
            {
                shutdown();
                // show ballon only when agent is away
                if (information_to_show) few.Show();
                    // let ballon visible
                else
                {
                    FormMain.current_form_main.Close();
                }
            }
        }

        


        /// <summary>
        /// configuration mode
        /// </summary>
        public void event_dates_configuration()
        {
            // restore shortcut in startup menu if missing
            if (this.options.shortcut_launch)
                this.create_or_restore_shortcut();
            else
                this.create_or_restore_windows_task(false);
            
            event_dates_check(false);

            // initialize components
            initialize_listview();
            initialize_outlookbar();
            initialize_month_calendar_context();

            // add events and messages to interface
            foreach (single_event se in clevents_to_remind.al_events)
            {
                // add to calendar
                add_event_to_calendar(se.date);
                // add to listview
                add_event_to_list_view(se);
            }
            foreach (string s in clwelcome_messages.messages)
            {
                // add to listbox
                FormMain.current_form_main.listBox_welcome_messages.Items.Add(s);
            }
            // make interface correspond to settings
            FormMain.current_form_main.checkBox_message_if_no_event.Checked=options.play_message_if_noevent_at_windows_startup;
            FormMain.current_form_main.checkBox_welcome_message_at_startup.Checked=options.play_welcome_message;
            FormMain.current_form_main.textBox_no_event_in_the_following_days.Text=options.no_event_speech;
            FormMain.current_form_main.textBox_speech_n_days_before.Text=options.n_days_before_event;

        }

        #endregion

        #region event management (add edit remove; find with date)


        public void add_event_to_calendar(date_event date)
        {
            FormMain.current_form_main.monthCalendar.AddAnnuallyBoldedDate(new System.DateTime(System.DateTime.Now.Year,date.month,date.day));
            FormMain.current_form_main.monthCalendar.UpdateBoldedDates();
        }

        public void remove_event_from_calendar(date_event date)// remove only if there's no other events
        {
            // check if there's other events
            for(int cpt=0;cpt<clevents_to_remind.al_events.Count;cpt++)
            {
                // if other events exist exit function
                if (((single_event)clevents_to_remind.al_events[cpt]).date==date) return;
            }
            FormMain.current_form_main.monthCalendar.RemoveAnnuallyBoldedDate(new System.DateTime(System.DateTime.Now.Year,date.month,date.day));
            FormMain.current_form_main.monthCalendar.UpdateBoldedDates();
        }

        public void add_event_to_list_view(single_event the_event)
        {
            System.Windows.Forms.ListViewItem lvi = FormMain.current_form_main.listView_event.Items.Add(the_event.name);
            lvi.SubItems.Add(the_event.date.tostring());
            lvi.SubItems.Add(the_event.event_description);
        }

        public void remove_event_from_listview(single_event the_event) 
        {
            string str_date=the_event.date.tostring();
            foreach (System.Windows.Forms.ListViewItem it in FormMain.current_form_main.listView_event.Items)
            {
                if ((it.SubItems[0].Text==the_event.name) && (it.SubItems[1].Text==str_date) && (it.SubItems[2].Text==the_event.event_description))
                {
                    FormMain.current_form_main.listView_event.Items.Remove(it);
                    return;// there should be only one
                }
            }
        }

        /// <summary>
        /// used only if removed from calendar
        /// removes all events of the day
        /// </summary>
        public void remove_event_from_listview(date_event date) 
        {
            string str_date=date.tostring();
            foreach (System.Windows.Forms.ListViewItem it in FormMain.current_form_main.listView_event.Items)
            {
                if (it.SubItems[1].Text==str_date) FormMain.current_form_main.listView_event.Items.Remove(it);
            }
        }

        /// <summary>
        /// if doubles return the first founded
        /// </summary>
        public single_event find_event_with_name_date_event(string name,date_event date,string str_event)
        {
            System.Collections.ArrayList al=new System.Collections.ArrayList(1);

            single_event se;
            for(int cpt=0;cpt<clevents_to_remind.al_events.Count;cpt++)
            {
                se=(single_event)clevents_to_remind.al_events[cpt];
                if ((se.date==date) && (se.name==name) && (se.event_description==str_event)) return se;
            }
            return new single_event();
        }
        /// <summary>
        /// if doubles return the first founded
        /// </summary>
        public single_event find_event_with_name_date_event(string name,string date,string str_event)
        {
            int month;
            int day;
            int pos=date.IndexOf("/");
            try
            {
                month=System.Convert.ToInt32(date.Substring(0,pos));
                day=System.Convert.ToInt32(date.Substring(pos+1));
                return find_event_with_name_date_event(name,new date_event(month,day),str_event);
            }
            catch // bad date replaced by 1/1
            {
                return new single_event();
            }
        }


        /// <summary>
        /// used with calendar view
        /// </summary>
        public single_event[] find_event_with_date(date_event date)
        {
            System.Collections.ArrayList al=new System.Collections.ArrayList(1);

            for(int cpt=0;cpt<clevents_to_remind.al_events.Count;cpt++)
            {
                if (((single_event)clevents_to_remind.al_events[cpt]).date==date) al.Add(clevents_to_remind.al_events[cpt]);
            }
            return (single_event[]) al.ToArray(typeof(single_event));
        }

        /// <summary>
        /// used only with calendar view
        /// </summary>
        public void remove_event(date_event date)
        {
            single_event[] array_events;
            array_events=find_event_with_date(date);
            
            for(int cpt=0;cpt<array_events.Length;cpt++)
            {
                // remove from memory must be before remove from interface
                clevents_to_remind.al_events.Remove(array_events[cpt]);
                // remove from interface (listview and calendar)
                remove_event_from_listview(date);
            }
            // remove event for calendar
            remove_event_from_calendar(date);
            // remove from database
            clevents_to_remind.save();
        }

        public void remove_event(single_event the_event)
        {
            clevents_to_remind.al_events.Remove(the_event);
            // remove from interface (listview and calendar)
            remove_event_from_listview(the_event);
            // remove event for calendar
            remove_event_from_calendar(the_event.date);
            // remove from database
            clevents_to_remind.save();
        }

        public void add_event(single_event the_event)
        {
            // add to memory
            clevents_to_remind.al_events.Add(the_event);
            // add to calendar
            add_event_to_calendar(the_event.date);
            // add to listview
            add_event_to_list_view(the_event);
            // add to database
            clevents_to_remind.save();
        }


        #endregion

        #region outlookbar initialization

        private void initialize_outlookbar()
        {
            FormMain.current_form_main.outlookbar=new UtilityLibrary.WinControls.OutlookBar();
            // add bands
            UtilityLibrary.WinControls.OutlookBarBand band_option = new UtilityLibrary.WinControls.OutlookBarBand("Options");
            UtilityLibrary.WinControls.OutlookBarBand band_help_about = new UtilityLibrary.WinControls.OutlookBarBand("Help");
            // setting band_option 
            // band_option.SmallImageList = FormMain.current_form_main.imageList_icons;
            band_option.LargeImageList = FormMain.current_form_main.imageList_big_icones;
            band_option.Items.Add(new UtilityLibrary.WinControls.OutlookBarItem("Calendar", 0));
            band_option.Items.Add(new UtilityLibrary.WinControls.OutlookBarItem("Events List", 1));

            band_option.Items.Add(new UtilityLibrary.WinControls.OutlookBarItem("Voice Options", 2));
            
            band_option.Items.Add(new UtilityLibrary.WinControls.OutlookBarItem("Startup Options", 5));
            band_option.Background = System.Drawing.SystemColors.AppWorkspace;
            band_option.TextColor = System.Drawing.Color.White;
            // setting band_help_about 
            // band_help_about.SmallImageList = FormMain.current_form_main.imageList_icons;
            band_help_about.LargeImageList = FormMain.current_form_main.imageList_big_icones;
            band_help_about.Items.Add(new UtilityLibrary.WinControls.OutlookBarItem("Voice Help", 3));
            band_help_about.Items.Add(new UtilityLibrary.WinControls.OutlookBarItem("About", 4));
            band_help_about.Background = System.Drawing.SystemColors.AppWorkspace;
            band_help_about.TextColor = System.Drawing.Color.White;

            // add bands
            FormMain.current_form_main.outlookbar.ContextMenu=null; // avoid to have a contexte menu
            FormMain.current_form_main.outlookbar.Bands.Add(band_option);
            FormMain.current_form_main.outlookbar.Bands.Add(band_help_about);

            FormMain.current_form_main.outlookbar.Dock =System.Windows.Forms.DockStyle.Fill;
            FormMain.current_form_main.outlookbar.SetCurrentBand(0);
            //FormMain.current_form_main.outlookbar.FlatArrowButtons = true;
            // add the control
            FormMain.current_form_main.panel_outlookbar.Controls.AddRange(new System.Windows.Forms.Control[]{FormMain.current_form_main.outlookbar});
        }

        #endregion

        #region listview initialization

        private void initialize_listview()
        {
            // create new ListViewEx control
            FormMain.current_form_main.listView_event=new UtilityLibrary.WinControls.ListViewEx();

            // create columns
            System.Windows.Forms.ColumnHeader ColumnName;
            System.Windows.Forms.ColumnHeader ColumnDate;
            System.Windows.Forms.ColumnHeader ColumnEvent;
            ColumnName = new System.Windows.Forms.ColumnHeader();
            ColumnDate = new System.Windows.Forms.ColumnHeader();
            ColumnEvent = new System.Windows.Forms.ColumnHeader();


            FormMain.current_form_main.listView_event.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] 
                            {
                                ColumnName,
                                ColumnDate,
                                ColumnEvent
                            });
            

            // setting text for columns headers
            ColumnName.Text="Name";
            ColumnDate.Text="Date";
            ColumnEvent.Text="Event";

            // setting icons for columns headers
            FormMain.current_form_main.listView_event.HeaderImageList = FormMain.current_form_main.imageList_icons;
            FormMain.current_form_main.listView_event.SetHeaderIcon(0,0);
            FormMain.current_form_main.listView_event.SetHeaderIcon(1,1);
            FormMain.current_form_main.listView_event.SetHeaderIcon(2,2);

            // setting the view of the window
            FormMain.current_form_main.listView_event.View=System.Windows.Forms.View.Details;
            FormMain.current_form_main.listView_event.GridLines=true;
            FormMain.current_form_main.listView_event.Dock =System.Windows.Forms.DockStyle.Fill;
            FormMain.current_form_main.listView_event.BorderStyle = System.Windows.Forms.BorderStyle.None;
            // setting size for columns headers
            //int size=FormMain.current_form_main.listView_event.Width/3;
            int size=FormMain.current_form_main.panel_listview.Width/3;// because of DockStyle.Fill
            ColumnName.Width =size;
            ColumnDate.Width=size;
            ColumnEvent.Width=size;
            // setting order options
            FormMain.current_form_main.listView_event.Sorting=System.Windows.Forms.SortOrder.Ascending;
            FormMain.current_form_main.listView_event.InitialSortedColumn = 0;
            FormMain.current_form_main.listView_event.SetColumnSortFormat(0,UtilityLibrary.WinControls.SortedListViewFormatType.String);
            FormMain.current_form_main.listView_event.SetColumnSortFormat(2,UtilityLibrary.WinControls.SortedListViewFormatType.String);//Date); error with date type
            FormMain.current_form_main.listView_event.SetColumnSortFormat(3,UtilityLibrary.WinControls.SortedListViewFormatType.String);
            // add the control to the form
            FormMain.current_form_main.panel_listview.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                                                               FormMain.current_form_main.listView_event});
        }
        #endregion 

        #region shortcut

        /// <summary>
        /// check if shortcut need to be recreated in startup of the start menu
        /// </summary>
        public void create_or_restore_shortcut()
        {
			try
			{
				if (!(System.IO.File.Exists(get_shortcut_name()))) 
					create_shortcut();
#if OLD_OS_CONFIG
                System.OperatingSystem osInfo = System.Environment.OSVersion;
                if (osInfo.Version.Major >= 6)
                {
#endif
                    TaskManager.DeleteTask(class_options.task_name);
#if OLD_OS_CONFIG
                }
                else
                {
                    int iret;
                    iret = Easy_task_delete(class_options.task_name);
                    if (iret != 0)
                    {
                        System.Windows.Forms.MessageBox.Show("Error during the deletion of task",
                            "Error",
                            System.Windows.Forms.MessageBoxButtons.OK,
                            System.Windows.Forms.MessageBoxIcon.Error);
                    }
                }
#endif
            }
			catch (Exception e)
			{
				System.Windows.Forms.MessageBox.Show( e.Message,
					"Error",
					System.Windows.Forms.MessageBoxButtons.OK,
					System.Windows.Forms.MessageBoxIcon.Error);
			}
        }
        /// <summary>
        /// check if shortcut need to be recreated in startup of the start menu
        /// </summary>
        public bool create_or_restore_windows_task(bool bShowErrorMessage)
        {
#if OLD_OS_CONFIG
            System.OperatingSystem osInfo = System.Environment.OSVersion;
            if (osInfo.Version.Major >= 6)
            {
#endif
                if (!TaskManager.CreateTask(class_options.task_name, 
                                            class_options.task_name, 
                                            System.Windows.Forms.Application.ExecutablePath,
                                            " /run", 
                                            (short)this.options.task_day_interval, 
                                            this.options.task_start_hour, 
                                            this.options.task_start_min,
                                            bShowErrorMessage
                                            )
                    )
                    return false;
#if OLD_OS_CONFIG
            }
            else
            {
                System.Exception ex = new System.Exception();
                int iret;
                
                string exename = new System.IO.FileInfo(System.Windows.Forms.Application.ExecutablePath).DirectoryName + "\\EvtReminderITask.exe";

                try
                {

                    iret = Easy_task_set_application(class_options.task_name, exename);
                    if (iret != 0)
                        throw ex;
                    iret = Easy_task_enable_task_for_logged_user(class_options.task_name);
                    if (iret != 0)
                        throw ex;

                    iret = Easy_task_set_daily_trigger(class_options.task_name, this.options.task_start_hour, this.options.task_start_min, this.options.task_day_interval);
                    if (iret != 0)
                        throw ex;

                }
                catch (System.Exception)
                {
                    if (bShowErrorMessage)
                    {
                        System.Windows.Forms.MessageBox.Show("Error during the creation of task.\r\nMake sure you have administrator rights",
                            "Error",
                            System.Windows.Forms.MessageBoxButtons.OK,
                            System.Windows.Forms.MessageBoxIcon.Error);
                    }
                    return false;
                }

            } // end of os version checking
#endif
            // at this point task creation should have succeeded
            // --> remove shortcut
            string shortcut_name;
            shortcut_name = get_shortcut_name();
            if (System.IO.File.Exists(shortcut_name))
                System.IO.File.Delete(shortcut_name);
            return true;

        }

        private static string get_folder_name()
        {
            string exename = System.Windows.Forms.Application.ExecutablePath;
            exename = exename.Replace('/', '\\'); // ExecutablePath is happy to return a mix between '/' and '\\' ... thank you ms for this buggy stuff
            return exename.Substring(0, exename.LastIndexOf("\\"));
        }
        private static string get_exe_name()
        {
            string exename = System.Windows.Forms.Application.ExecutablePath;
            exename = exename.Replace('/', '\\'); // ExecutablePath is happy to return a mix between '/' and '\\' ... thank you ms for this buggy stuff
            return exename.Substring(exename.LastIndexOf("\\"));
        }

        /// <summary>
        /// get file name for putting the shortcut in startup of the start menu
        /// </summary>
        /// <returns></returns>
        private string get_shortcut_name()
        {
            string exename=System.Windows.Forms.Application.ExecutablePath;
            // set the outpout for the shortcut
            string link_name = get_exe_name();
            link_name=link_name.Substring(0,link_name.LastIndexOf("."))+".lnk";
            link_name=System.Environment.GetFolderPath(System.Environment.SpecialFolder.Startup)+link_name;
            return link_name;
        }

        /// <summary>
        /// create shortcut with the /run argument and put it in startup of the start menu
        /// </summary>
        private void create_shortcut()
        {
            string exename=System.Windows.Forms.Application.ExecutablePath;
            string working_dir=get_folder_name();

            IShellLink link = (IShellLink)new ShellLink();

            // setup shortcut information
            link.SetPath(exename);
            link.SetDescription("Event Reminder");
            link.SetWorkingDirectory(working_dir);
            link.SetArguments("/run");

            // save it
            IPersistFile file = (IPersistFile)link;
            file.Save(get_shortcut_name(), false);
        }


        /// <summary>
        /// remove shortcut
        /// </summary>
        private void remove_shortcut()
        {
            string shortcut=get_shortcut_name();
            if (System.IO.File.Exists(shortcut)) System.IO.File.Delete(shortcut);
        }
        #endregion

        #region monthcalendar popup menu 
        
        private void initialize_month_calendar_context()
        {
            // create menu and handlers
            System.Windows.Forms.MenuItem[] menu=new System.Windows.Forms.MenuItem[3];
            UtilityLibrary.Menus.MenuItemEx menuex_add=new UtilityLibrary.Menus.MenuItemEx("Add Event",new EventHandler(menu_add_event));
            menuex_add.Icon = new System.Drawing.Bitmap(FormMain.current_form_main.imageList_menu.Images[0]);
            menu[0]=(System.Windows.Forms.MenuItem)menuex_add;
            UtilityLibrary.Menus.MenuItemEx menuex_edit=new UtilityLibrary.Menus.MenuItemEx("Edit Selected Event(s)",new EventHandler(menu_edit_event));
            menuex_edit.Icon = new System.Drawing.Bitmap(FormMain.current_form_main.imageList_menu.Images[1]);
            menu[1]=(System.Windows.Forms.MenuItem)menuex_edit;
            UtilityLibrary.Menus.MenuItemEx menuex_remove=new UtilityLibrary.Menus.MenuItemEx("Remove Selected Event(s)",new EventHandler(menu_remove_event));
            menuex_remove.Icon = new System.Drawing.Bitmap(FormMain.current_form_main.imageList_menu.Images[2]);
            menu[2]=(System.Windows.Forms.MenuItem)menuex_remove;

            // attach menu to monthcalendar
            FormMain.current_form_main.monthCalendar.ContextMenu=new System.Windows.Forms.ContextMenu(menu);
        }

        private void menu_add_event(object sender, System.EventArgs e)
        {
            FormEventEdit fee=new FormEventEdit();
            fee.ShowDialog();
        }
        private void menu_edit_event(object sender, System.EventArgs e)
        {
            FormEventEdit fee=new FormEventEdit();
            // find event
            single_event[] array_single_event;
            System.DateTime dt= FormMain.current_form_main.monthCalendar.SelectionStart;
            array_single_event=find_event_with_date(new date_event(dt.Month,dt.Day));
            // show dialog
            if (array_single_event.Length>0)// if events found
                fee.event_to_edit(array_single_event);
            // else it just ad event
            fee.ShowDialog();
        }
        private void menu_remove_event(object sender, System.EventArgs e)
        {
            System.DateTime dt= FormMain.current_form_main.monthCalendar.SelectionStart;
            remove_event(new date_event(dt.Month,dt.Day));
        }

        #endregion

        private string get_random_welcome_message()
        {
            if (clwelcome_messages.messages.Count==0) return "";
            int i;
            System.Random r = new System.Random(System.DateTime.Now.Millisecond + System.DateTime.Now.Day + System.DateTime.Now.Day + System.DateTime.Now.Day);
            i=r.Next(clwelcome_messages.messages.Count);
            return ((string)clwelcome_messages.messages[i])+".";
        }

        #region agent animation

        private void play_random_search()
        {
            //string[] anims={"Read","Process","Search"};
            //play_random_animation(anims);

            // play read only and not random with search and process because these two animations are too long
            clagent.play("Read");
            clagent.play("ReadReturn");
        }

        private void play_random_event_animation()
        {
            string[] anims={"DoMagic1","Announce","DoMagic2","Congratulate"};
            play_random_animation(anims);
        }

        private void play_random_attention_animation()
        {
            string[] anims={"GetAttention","Surprised"};
            play_random_animation(anims);
        }

        private void play_random_no_event_animation()
        {
            string[] anims={"Confused","Decline"};
            play_random_animation(anims);
        }
        private void play_random_move()
        {
            // get windows size
            System.Drawing.Rectangle working_area = System.Windows.Forms.SystemInformation.WorkingArea;
            int x;
            int y;
            System.Random r=new System.Random();
            x=r.Next(working_area.Right/2);
            y=r.Next(working_area.Bottom/2);
            
            clagent.obj_char_ex.Top=(short)(y+working_area.Bottom/4);
            clagent.obj_char_ex.Left=(short)(x+working_area.Right/4);


            // move to is quite too long
        }
        private void play_random_animation(string[] animations)
        {
            if (animations.Length==0) return;
            int i;
            System.Random r = new System.Random(System.DateTime.Now.Millisecond + System.DateTime.Now.Day);
            i=r.Next(animations.Length);
            try
            {
                clagent.play(animations[i]);
                if (animations[i] == "GetAttention") clagent.play("GetAttentionReturn");
                clagent.play("RestPose");
            }
            catch (System.Exception ex)
            {
            	
            }

        }
        #endregion

    }
}