#include <windows.h>
#include "HotFixes.h"

BOOL ExecuteSubInstall(TCHAR* CommandLine)
{
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
    if (::CreateProcess(0,CommandLine,0,0,0,0,0,0,&si,&pi))
    {
        // Wait until child process exits.
        ::WaitForSingleObject( pi.hProcess, INFINITE );

        // Close process and thread handles. 
        ::CloseHandle( pi.hProcess );
        ::CloseHandle( pi.hThread );
        return TRUE;
    }
    else
    {
        TCHAR Msg[2*MAX_PATH];

        _sntprintf(Msg,2*MAX_PATH, _T("Error launching %s.\r\nInstall cancelled."),CommandLine);
        ::MessageBox(0,Msg,_T("Error"),MB_ICONERROR);

        // Close process and thread handles. 
        ::CloseHandle( pi.hProcess );
        ::CloseHandle( pi.hThread );
        return FALSE;
    }
}

int WINAPI WinMain(HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
    )
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);
    ::CoInitialize(0);

    ////////////////////////
    // get exe path
    ////////////////////////
    TCHAR CurrentPath[MAX_PATH];
    // get exe full path
    ::GetModuleFileName(NULL,CurrentPath,MAX_PATH);
    // remove exe name
    TCHAR* psz = _tcsrchr(CurrentPath,'\\');
    if (psz)
    {
        psz++;
        *psz=0;
    }

    ////////////////////////
    // Get the Windows version.
    ////////////////////////
    DWORD dwVersion = ::GetVersion();
    DWORD dwMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
    DWORD dwMinorVersion = (DWORD)(HIBYTE(LOWORD(dwVersion)));

    // dummy check
    if (dwMajorVersion<6)
    {
        ::MessageBox(0,
                    _T("This setup reserved for Vista and Seven.\r\n")
                    _T("For previous OS, please use another setup available at http://jacquelin.potier.free.fr/eventreminder/"),
                    _T("Error"),
                    MB_ICONERROR
                    );

        return -1;
    }

    ////////////////////////
    // if windows 7 (agent supposed to be installed on Vista)
    ////////////////////////
    if ( (dwMajorVersion==6) && (dwMinorVersion>=1) )
    {
        // if hotfix not installed
        if (!CHotFixes::IsHotFixInstalled(_T("KB969168")))
        {
            TCHAR AgentInstallPath[MAX_PATH];
            _tcscpy(AgentInstallPath,CurrentPath);

            // switch platform (32 or 64)
            SYSTEM_INFO SystemInfo={0};
            ::GetNativeSystemInfo(&SystemInfo);
            if (SystemInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
            {
                // install 64 bit hotfix
                _tcscat(AgentInstallPath,_T("Ms Agent controls\\Windows6.1-KB969168-x64.msu"));
            }
            else
            {
                // install 32 bit hotfix
                _tcscat(AgentInstallPath,_T("Ms Agent controls\\Windows6.1-KB969168-x86.msu"));
            }

            // msu dirty install because shellexecute doesn't work
            TCHAR FullCommandLine[3*MAX_PATH];
            ::GetSystemDirectory(FullCommandLine,MAX_PATH);
            _tcscat(FullCommandLine,_T("\\wusa.exe \""));
            _tcscat(FullCommandLine,AgentInstallPath);
            _tcscat(FullCommandLine,_T("\" /quiet /norestart"));
            if (!ExecuteSubInstall(FullCommandLine))
            {
                ::CoUninitialize();
                return -1;
            }
        }
        
    }
    // install an agent in quiet mode
    TCHAR AgentPath[MAX_PATH];
    _tcscpy(AgentPath,CurrentPath);
    _tcscat(AgentPath,_T("Ms Agent controls\\Merlin.exe /Q"));

    // run setup common to Vista / Seven
    TCHAR SetupPath[MAX_PATH];
    _tcscpy(SetupPath,CurrentPath);
    _tcscat(SetupPath,_T("SetupEventReminder\\Setup.exe"));
    if (ExecuteSubInstall(SetupPath))
    {
        ::MessageBox(0,
                    _T("Installation successfully completed\r\n")
                    _T("You may need to change the installation directory right access to allow standard users to save options\r\n"),
                    _T("Information"),
                    MB_ICONINFORMATION);
    }

    ::CoUninitialize();
    return 0;
}