/*
Copyright (C) 2020 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2017 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#ifndef _WIN32_IE
    #define _WIN32_IE 0x0500 // should be at least ie 5
#endif

// Enable Visual Style
#if defined _AMD64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='amd64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#elif defined _WIN64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='ia64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#else
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif


#include <Windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include <Commctrl.h>
#pragma comment(lib,"Comctl32.lib")

#include "DialogBase.h"
#include "resource.h"

class CDialogSetup:public CDialogBase
{
public:
    CDialogSetup():CDialogBase()
    {
        this->GetExePath();
    }

protected:

    TCHAR CurrentPath[MAX_PATH];

    void GetExePath()
    {
        // get exe full path
        ::GetModuleFileName(NULL,CurrentPath,MAX_PATH);
        // remove exe name (get upper dir)
        TCHAR* psz1 = _tcsrchr(CurrentPath,'\\');
        TCHAR* psz2 = _tcsrchr(CurrentPath,'/');
        TCHAR* psz = __max(psz1,psz2);
        if (psz)
        {
            psz++; // keep '\\'
            *psz=0;
        }
    }

    BOOL ExecuteSubInstall(TCHAR* CommandLine)
    {
        STARTUPINFO si;
        PROCESS_INFORMATION pi;
        ZeroMemory( &si, sizeof(si) );
        si.cb = sizeof(si);
        ZeroMemory( &pi, sizeof(pi) );
        if (::CreateProcess(0,CommandLine,0,0,0,0,0,0,&si,&pi))
        {
            // Wait until child process exits.
            ::WaitForSingleObject( pi.hProcess, INFINITE );

            // Close process and thread handles. 
            ::CloseHandle( pi.hProcess );
            ::CloseHandle( pi.hThread );
            return TRUE;
        }
        else
        {
            TCHAR Msg[2*MAX_PATH];

            _sntprintf(Msg,2*MAX_PATH, _T("Error launching %s.\r\nInstall cancelled."),CommandLine);
            ::MessageBox(0,Msg,_T("Error"),MB_ICONERROR);

            // Close process and thread handles. 
            ::CloseHandle( pi.hProcess );
            ::CloseHandle( pi.hThread );
            return FALSE;
        }
    }

    BOOL InstallAgent()
    {
        TCHAR AgentInstallPath[MAX_PATH];
        _tcscpy(AgentInstallPath,CurrentPath);

        // install 64 bit hotfix
        _tcscat(AgentInstallPath,_T("Ms Agent controls\\DoubleAgent_x64x86.msi"));

        // msi install through msiexec.exe because shellexecute doesn't work
        TCHAR FullCommandLine[3*MAX_PATH];
        TCHAR Dir[MAX_PATH];
        ::GetSystemDirectory(Dir,_countof(Dir));
        _sntprintf(FullCommandLine,_countof(FullCommandLine),TEXT("%s\\msiexec.exe /i \"%s\""),Dir,AgentInstallPath);
        FullCommandLine[_countof(FullCommandLine)-1] = 0;
        return ExecuteSubInstall(FullCommandLine);   
    }

    BOOL InstallEventReminder()
    {
        // run setup 
        TCHAR SetupPath[MAX_PATH];
        _tcscpy(SetupPath,CurrentPath);
        _tcscat(SetupPath,_T("SetupEventReminder\\Setup.exe"));
        BOOL bRetValue = ExecuteSubInstall(SetupPath);
        if (bRetValue)
        {
            ::MessageBox(0,
                _T("Installation successfully completed\r\n")
                _T("Warning : You need to change the installation directory rights access to allow users to save options\r\n"),
                _T("Information"),
                MB_ICONINFORMATION);
        }
        return bRetValue;
    }

    virtual void OnCommand(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(lParam);
        switch(LOWORD(wParam))
        {
        case IDC_BUTTON_INSTALL_AGENT:
            this->InstallAgent();
            break;
        case IDC_BUTTON_INSTALL_SPEAKING_EVENT_REMINDER:
            this->InstallEventReminder();
            break;
        }
    }
};



int WINAPI WinMain(HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
    )
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    // enable Xp style
    InitCommonControls();
    ::CoInitialize(0);

    ////////////////////////
    // Get the Windows version.
    ////////////////////////
    DWORD dwVersion = ::GetVersion();
    DWORD dwMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
    DWORD dwMinorVersion = (DWORD)(HIBYTE(LOWORD(dwVersion)));

    /*
    +------------------------------------------------------------------------------+
    |                    |   PlatformID    |   Major version   |   Minor version   |
    +------------------------------------------------------------------------------+
    | Windows 95         |  Win32Windows   |         4         |          0        |
    | Windows 98         |  Win32Windows   |         4         |         10        |
    | Windows Me         |  Win32Windows   |         4         |         90        |
    | Windows NT 4.0     |  Win32NT        |         4         |          0        |
    | Windows 2000       |  Win32NT        |         5         |          0        |
    | Windows XP         |  Win32NT        |         5         |          1        |
    | Windows 2003       |  Win32NT        |         5         |          2        |
    | Windows Vista      |  Win32NT        |         6         |          0        |
    | Windows 2008       |  Win32NT        |         6         |          0        |
    | Windows 7          |  Win32NT        |         6         |          1        |
    | Windows 2008 R2    |  Win32NT        |         6         |          1        |
    | Windows 8          |  Win32NT        |         6         |          2        |
    | Windows 8.1        |  Win32NT        |         6         |          3        |
    +------------------------------------------------------------------------------+
    | Windows 10         |  Win32NT        |        10         |          0        |
    +------------------------------------------------------------------------------+
    */
    if ( (dwMajorVersion < 6) || 
         ( (dwMajorVersion == 6) && dwMinorVersion <2 )
         )
    {
        ::MessageBox(0,
                    _T("This setup reserved for Win 8, Win 10 and Win 11.\r\n")
                    _T("For previous OS, please use another setup available at http://jacquelin.potier.free.fr/eventreminder/"),
                    _T("Error"),
                    MB_ICONERROR
                    );

        goto CleanUp;
    }

    {
        CDialogSetup DialogSetup;
        DialogSetup.Show(hInstance,NULL,IDD_DIALOG_SETUP,IDI_ICON_APP);
    }

CleanUp:
    ::CoUninitialize();
    return 0;
}

